@@ -12,6 +12,7 @@ except ImportError:
 from peewee import ImproperlyConfigured
 from peewee import Insert
 from peewee import MySQLDatabase
+from peewee import Node
 from peewee import NodeList
 from peewee import SQL
 from peewee import TextField
@@ -36,6 +37,9 @@ class MySQLConnectorDatabase(MySQLDatabase):
                 raise InterfaceError('Error, database connection not opened.')
         return self._state.conn.cursor(buffered=True)
 
+    def get_binary_type(self):
+        return mysql_connector.Binary
+
 
 class MariaDBConnectorDatabase(MySQLDatabase):
     def _connect(self):
@@ -75,17 +79,33 @@ class MariaDBConnectorDatabase(MySQLDatabase):
                 return cursor.lastrowid
         return cursor
 
+    def get_binary_type(self):
+        return mariadb.Binary
+
 
 class JSONField(TextField):
     field_type = 'JSON'
 
-    def db_value(self, value):
-        if value is not None:
-            return json.dumps(value)
+    def __init__(self, json_dumps=None, json_loads=None, **kwargs):
+        self._json_dumps = json_dumps or json.dumps
+        self._json_loads = json_loads or json.loads
+        super(JSONField, self).__init__(**kwargs)
 
     def python_value(self, value):
         if value is not None:
-            return json.loads(value)
+            try:
+                return self._json_loads(value)
+            except (TypeError, ValueError):
+                return value
+
+    def db_value(self, value):
+        if value is not None:
+            if not isinstance(value, Node):
+                value = self._json_dumps(value)
+            return value
+
+    def extract(self, path):
+        return fn.json_extract(self, path)
 
 
 def Match(columns, expr, modifier=None):
