@@ -34,7 +34,6 @@ That's it!
 import functools
 import heapq
 import logging
-import random
 import threading
 import time
 from collections import namedtuple
@@ -48,6 +47,10 @@ except ImportError:
     TRANSACTION_STATUS_IDLE = \
             TRANSACTION_STATUS_INERROR = \
             TRANSACTION_STATUS_UNKNOWN = None
+try:
+    from psycopg.pq import TransactionStatus
+except ImportError:
+    pass
 
 from peewee import MySQLDatabase
 from peewee import PostgresqlDatabase
@@ -68,6 +71,10 @@ class MaxConnectionsExceeded(ValueError): pass
 PoolConnection = namedtuple('PoolConnection', ('timestamp', 'connection',
                                                'checked_out'))
 
+class _sentinel(object):
+    def __lt__(self, other):
+        return True
+
 
 def locked(fn):
     @functools.wraps(fn)
@@ -136,7 +143,8 @@ class PooledDatabase(object):
         while True:
             try:
                 # Remove the oldest connection from the heap.
-                ts, conn = heapq.heappop(self._connections)
+                ts, _, c_conn = heapq.heappop(self._connections)
+                conn = c_conn
                 key = self.conn_key(conn)
             except IndexError:
                 ts = conn = None
@@ -198,7 +206,8 @@ class PooledDatabase(object):
                 super(PooledDatabase, self)._close(conn)
             elif self._can_reuse(conn):
                 logger.debug('Returning %s to pool.', key)
-                heapq.heappush(self._connections, (pool_conn.timestamp, conn))
+                heapq.heappush(self._connections,
+                               (pool_conn.timestamp, _sentinel(), conn))
             else:
                 logger.debug('Closed %s.', key)
 
@@ -224,7 +233,7 @@ class PooledDatabase(object):
     @locked
     def close_idle(self):
         # Close any open connections that are not currently in-use.
-        for _, conn in self._connections:
+        for _, _, conn in self._connections:
             self._close(conn, close_conn=True)
         self._connections = []
 
@@ -249,7 +258,7 @@ class PooledDatabase(object):
         # Close all connections -- available and in-use. Warning: may break any
         # active connections used by other threads.
         self.close()
-        for _, conn in self._connections:
+        for _, _, conn in self._connections:
             self._close(conn, close_conn=True)
         for pool_conn in self._in_use.values():
             self._close(pool_conn.connection, close_conn=True)
@@ -317,9 +326,9 @@ try:
                 return True
 
             txn_status = conn.pgconn.transaction_status
-            if txn_status == conn.TransactionStatus.UNKNOWN:
+            if txn_status == TransactionStatus.UNKNOWN:
                 return True
-            elif txn_status != conn.TransactionStatus.IDLE:
+            elif txn_status != TransactionStatus.IDLE:
                 conn.rollback()
             return False
 
@@ -328,11 +337,11 @@ try:
             # Do not return connection in an error state, as subsequent queries
             # will all fail. If the status is unknown then we lost the connection
             # to the server and the connection should not be re-used.
-            if txn_status == conn.TransactionStatus.UNKNOWN:
+            if txn_status == TransactionStatus.UNKNOWN:
                 return False
-            elif txn_status == conn.TransactionStatus.INERROR:
+            elif txn_status == TransactionStatus.INERROR:
                 conn.reset()
-            elif txn_status != conn.TransactionStatus.IDLE:
+            elif txn_status != TransactionStatus.IDLE:
                 conn.rollback()
             return True
 except ImportError:
